#include "amxxmodule.h"
#include <VoiceTranscoderAPI.h>
#include <Module.h>
#include <rehlds_api.h>

IRehldsApi *g_pRehldsApi;
IVoiceTranscoderAPI *g_pVoiceTranscoderApi;

int g_forward_ClientStartSpeak;
int g_forward_ClientStopSpeak;

static cell AMX_NATIVE_CALL VTC_IsClientSpeaking(AMX *amx, cell *params) {
	return g_pVoiceTranscoderApi->IsClientSpeaking((size_t)params[1]);
}

static cell AMX_NATIVE_CALL VTC_MuteClient(AMX *amx, cell *params) {
	g_pVoiceTranscoderApi->MuteClient((size_t)params[1]);

	return 0;
}

static cell AMX_NATIVE_CALL VTC_UnmuteClient(AMX *amx, cell *params) {
	g_pVoiceTranscoderApi->UnmuteClient((size_t)params[1]);

	return 0;
}

AMX_NATIVE_INFO g_natives_VoiceTranscoder[] = {
	{"VTC_IsClientSpeaking", &VTC_IsClientSpeaking},
	{"VTC_MuteClient", &VTC_MuteClient},
	{"VTC_UnmuteClient", &VTC_UnmuteClient},
	{nullptr, nullptr}
};

void OnClientStartSpeak(size_t clientIndex) {
	MF_ExecuteForward(g_forward_ClientStartSpeak, clientIndex);
}

void OnClientStopSpeak(size_t clientIndex) {
	MF_ExecuteForward(g_forward_ClientStopSpeak, clientIndex);
}

IRehldsApi *GetRehldsAPI(const Module &engineModule) {
	void *(*CreateInterface)(const char *interfaceName, size_t *returnCode) = engineModule.FindSymbol("CreateInterface");

	if (CreateInterface == nullptr) {
		return nullptr;
	}

	return (AnyPointer)CreateInterface("VREHLDS_HLDS_API_VERSION001", nullptr);
}

void OnAmxxAttach() {
	Module engineModule(g_engfuncs.pfnPrecacheModel);

	g_pRehldsApi = GetRehldsAPI(engineModule);

	if (g_pRehldsApi != nullptr) {
		if (g_pRehldsApi->GetMajorVersion() != REHLDS_API_VERSION_MAJOR) {
			MF_Log("Unexpected ReHLDS API Major version (current %u, expected %u)", g_pRehldsApi->GetMajorVersion(), REHLDS_API_VERSION_MAJOR);
			return;
		}
		if (g_pRehldsApi->GetMinorVersion() < REHLDS_API_VERSION_MINOR) {
			MF_Log("Unexpected ReHLDS API Minor version (current %u, expected %u)", g_pRehldsApi->GetMinorVersion(), REHLDS_API_VERSION_MINOR);
			return;
		}

		g_pVoiceTranscoderApi = (AnyPointer)g_pRehldsApi->GetFuncs()->GetPluginApi("VoiceTranscoder");
	} else {
		g_pVoiceTranscoderApi = (AnyPointer)strtoul(CVAR_GET_STRING("VTC_API"), nullptr, 16);
	}

	if (g_pVoiceTranscoderApi == nullptr) {
		MF_Log("Can't get VoiceTranscoder API");
		return;
	}
	if (g_pVoiceTranscoderApi->GetMajorVersion() != VOICETRANSCODER_API_VERSION_MAJOR) {
		MF_Log("Unexpected VoiceTranscoder API Major version (current %u, expected %u)", g_pVoiceTranscoderApi->GetMajorVersion(), VOICETRANSCODER_API_VERSION_MAJOR);
		return;
	}
	if (g_pVoiceTranscoderApi->GetMinorVersion() < VOICETRANSCODER_API_VERSION_MINOR) {
		MF_Log("Unexpected VoiceTranscoder API Minor version (current %u, expected %u)", g_pVoiceTranscoderApi->GetMinorVersion(), VOICETRANSCODER_API_VERSION_MINOR);
		return;
	}

	g_pVoiceTranscoderApi->ClientStartSpeak()->RegisterCallback(&OnClientStartSpeak);
	g_pVoiceTranscoderApi->ClientStopSpeak()->RegisterCallback(&OnClientStopSpeak);

	MF_AddNatives(g_natives_VoiceTranscoder);
}

void OnAmxxDetach() {
	if (g_pVoiceTranscoderApi != nullptr) {
		g_pVoiceTranscoderApi->ClientStartSpeak()->UnregisterCallback(&OnClientStartSpeak);
		g_pVoiceTranscoderApi->ClientStopSpeak()->UnregisterCallback(&OnClientStopSpeak);
	}
}

void OnPluginsLoaded() {
	g_forward_ClientStartSpeak = MF_RegisterForward("VTC_OnClientStartSpeak", ET_IGNORE, FP_CELL, FP_DONE);
	g_forward_ClientStopSpeak = MF_RegisterForward("VTC_OnClientStopSpeak", ET_IGNORE, FP_CELL, FP_DONE);
}