#include "Module.h"
#ifdef __linux__
	#include <dlfcn.h> // dlopen, etc...
#endif

const Module::Handle Module::INVALID_HANDLE = (Module::Handle)0;

Module::Module() {
	m_handle = INVALID_HANDLE;
	m_baseAddress = (uintptr_t)nullptr;
}

Module::Module(AnyPointer insideAddress) {
	Open(insideAddress);
}

Module::~Module() {
	if (IsOpened()) {
		Close();
	}
}

void Module::Open(AnyPointer insideAddress) {
	if (IsOpened()) {
		Close();
	}

#ifdef _WIN32
	GetModuleHandleEx(GET_MODULE_HANDLE_EX_FLAG_UNCHANGED_REFCOUNT | GET_MODULE_HANDLE_EX_FLAG_FROM_ADDRESS, insideAddress, &m_handle);
	m_baseAddress = (uintptr_t)m_handle;
#elif defined __linux__
	Dl_info dlinfo;
	dladdr(insideAddress, &dlinfo);

	m_handle = dlopen(dlinfo.dli_fname, RTLD_NOW);
	m_baseAddress = (uintptr_t)dlinfo.dli_fbase;
#endif
}

void Module::Close() {
#ifdef __linux__
	dlclose(m_handle);
#endif
	m_handle = INVALID_HANDLE;
	m_baseAddress = (uintptr_t)nullptr;
}

bool Module::IsOpened() const {
	return m_handle != INVALID_HANDLE;
}

AnyPointer Module::FindSymbol(const char *symbolName) const {
#ifdef _WIN32
	return GetProcAddress(m_handle, symbolName);
#elif defined __linux__
	return dlsym(m_handle, symbolName);
#endif
}